/*

STANDARD.H  Custom header file that provides commonly used equates

*/

#ifndef STANDARD_H
#define	STANDARD_H

typedef	int BOOLEAN;

#define	byte		unsigned char
#define	word		unsigned short
#define	dword		unsigned long

#endif
