/***********************************************************************/
/*                                                                     */
/* Project      : GRAV                                                 */
/* Current Rel. : 3.1                                                  */
/* Creator      : Michael Knigge                                       */
/* Creation Date: 01/21/95                                             */
/* Description  : Header-File for module directory.c                   */
/*                                                                     */
/***********************************************************************/

int grav_check_filename(char *);
int grav_read_directory(char *, int *, DIRTAB []);
int grav_show_directory(int, int, int, DIRTAB[]);
int grav_change_directory(int *, int, DIRTAB[]);
int grav_sort_directory(void *, void *);
