/***********************************************************************/
/*                                                                     */
/* Project      : GRAV                                                 */
/* Current Rel. : 3.1                                                  */
/* Creator      : Michael Knigge                                       */
/* Creation Date: 02/06/95                                             */
/* Description  : header-file for effects.c                            */
/*                                                                     */
/***********************************************************************/

int grav_fade_out(GBM *, GBMRGB []);
int grav_fade_in(GBM *, GBMRGB []);
int grav_rotate_palette(GBM *, GBMRGB [], int);
int grav_invert(char *, GBM *, GBMRGB []);
int grav_invert_24bit(char *, GBM *, GBMRGB []);
int grav_mirror_image(GBM *, char *);
int grav_flip_image(GBM *, char *);
char *grav_to_grey(char *, GBM *, GBMRGB[]);
char *grav_scale(int *, char *, GBM *, int, int);

