/***********************************************************************/
/*                                                                     */
/* Project      : GRAV                                                 */
/* Current Rel. : 3.0                                                  */
/* Creator      : Michael Knigge                                       */
/* Creation Date: 01/30/95                                             */
/* Description  : printing files                                       */
/*                                                                     */
/* Functions    : grav_print_file()                                    */
/*                grav_print_eps_header()                              */
/*                grav_print_ps_header()                               */
/*                grav_print_image()                                   */
/*                grav_calculate_colors()                              */
/*                                                                     */
/***********************************************************************/

#include <stdio.h>
#include <strings.h>
#include <unistd.h>
#include <ncurses.h>
#include "standard.h"
#include "gbm.h"
#include "defines.h"
#include "keys.h"
#include "trans.h"
#include "read.h"
#include "error.h"
#include "print.h"
#include "view.h"

int grav_print_file()
{
   return(GRAV_NO_ERROR);
}
