/***********************************************************************/
/*                                                                     */
/* Project      : GRAV                                                 */
/* Current Rel. : 3.1                                                  */
/* Creator      : Michael Knigge                                       */
/* Creation Date: 01/21/95                                             */
/* Description  : Header-File for module misc.c                        */
/*                                                                     */
/***********************************************************************/

int grav_interpret_params(int, char *[], int *, int *);
int grav_init(int *, GMODE[]);
int grav_init_ncurses();
int grav_init_colors();
int grav_guess_filetype(char *, int *);
int grav_greetings(int);
int grav_goodbye(int);
int grav_cleanup();
int grav_change_mode(int *, int, GMODE[]);
int grav_display_mode(int, int, GMODE[]);
int grav_display_card(int, int);
int grav_reading_msg(int);
int grav_writing_msg(int);
int grav_show_help(int);
