/***********************************************************************/
/*                                                                     */
/* Project      : GRAV                                                 */
/* Current Rel. : 3.1                                                  */
/* Creator      : Michael Knigge                                       */
/* Creation Date: 01/30/95                                             */
/* Description  : Some defines for key-handling                        */
/*                                                                     */
/***********************************************************************/

/*
 * here we need to undef KEY_ENTER from ncurses
 */

#ifdef KEY_ENTER
#undef KEY_ENTER
#endif

#define MOUSE_MOVED       255      /* Indicate that mouse was moved    */

#define KEY_SPACE          32      /* ASCII-Code for Space             */
#define KEY_ENTER          10      /* ASCII-Code for Enter             */

#define KEY_gt             62      /* ASCII-Code for ">"               */
#define KEY_lt             60      /* ASCII-Code for "<"               */
#define KEY_eq             61      /* ASCII-Code for "="               */

#define KEY_a              97      /* ASCII-Code for "a"               */
#define KEY_b              98      /* ASCII-Code for "b"               */
#define KEY_c              99      /* ASCII-Code for "c"               */
#define KEY_d             100      /* ASCII-Code for "d"               */
#define KEY_e             101      /* ASCII-Code for "e"               */
#define KEY_f             102      /* ASCII-Code for "f"               */
#define KEY_g             103      /* ASCII-Code for "g"               */
#define KEY_h             104      /* ASCII-Code for "h"               */
#define KEY_i             105      /* ASCII-Code for "i"               */
#define KEY_j             106      /* ASCII-Code for "j"               */
#define KEY_k             107      /* ASCII-Code for "k"               */
#define KEY_l             108      /* ASCII-Code for "l"               */
#define KEY_m             109      /* ASCII-Code for "m"               */
#define KEY_n             110      /* ASCII-Code for "n"               */
#define KEY_o             111      /* ASCII-Code for "o"               */
#define KEY_p             112      /* ASCII-Code for "p"               */
#define KEY_q             113      /* ASCII-Code for "q"               */
#define KEY_r             114      /* ASCII-Code for "r"               */
#define KEY_s             115      /* ASCII-Code for "s"               */
#define KEY_t             116      /* ASCII-Code for "t"               */
#define KEY_u             117      /* ASCII-Code for "u"               */
#define KEY_v             118      /* ASCII-Code for "v"               */
#define KEY_w             119      /* ASCII-Code for "w"               */
#define KEY_x             120      /* ASCII-Code for "x"               */
#define KEY_y             121      /* ASCII-Code for "y"               */
#define KEY_z             122      /* ASCII-Code for "z"               */

#define KEY_A              65      /* ASCII-Code for "A"               */
#define KEY_B              66      /* ASCII-Code for "B"               */
#define KEY_C              67      /* ASCII-Code for "C"               */
#define KEY_D              68      /* ASCII-Code for "D"               */
#define KEY_E              69      /* ASCII-Code for "E"               */
#define KEY_G              71      /* ASCII-Code for "G"               */
#define KEY_H              72      /* ASCII-Code for "H"               */
#define KEY_I              73      /* ASCII-Code for "I"               */
#define KEY_J              74      /* ASCII-Code for "J"               */
#define KEY_K              75      /* ASCII-Code for "K"               */
#define KEY_L              76      /* ASCII-Code for "L"               */
#define KEY_M              77      /* ASCII-Code for "M"               */
#define KEY_N              78      /* ASCII-Code for "N"               */
#define KEY_O              79      /* ASCII-Code for "O"               */
#define KEY_P              80      /* ASCII-Code for "P"               */
#define KEY_Q              81      /* ASCII-Code for "Q"               */
#define KEY_R              82      /* ASCII-Code for "R"               */
#define KEY_S              83      /* ASCII-Code for "S"               */
#define KEY_T              84      /* ASCII-Code for "T"               */
#define KEY_U              85      /* ASCII-Code for "U"               */
#define KEY_V              86      /* ASCII-Code for "V"               */
#define KEY_W              87      /* ASCII-Code for "W"               */
#define KEY_X              88      /* ASCII-Code for "X"               */
#define KEY_Y              89      /* ASCII-Code for "Y"               */
#define KEY_Z              90      /* ASCII-Code for "Z"               */
