/*

GBMMIR.H  Interface to Mirror Image of General Bitmap

*/

extern BOOLEAN gbm_ref_vert(GBM *gbm, byte *data);
extern BOOLEAN gbm_ref_horz(GBM *gbm, byte *data);
extern void gbm_transpose(GBM *gbm, byte *data, byte *data_t);
