/***********************************************************************/
/*                                                                     */
/* Project      : GRAV                                                 */
/* Current Rel. : 3.1                                                  */
/* Creator      : Michael Knigge                                       */
/* Creation Date: 01/30/95                                             */
/* Description  : Header-File for module view.c                        */
/*                                                                     */
/***********************************************************************/

int  grav_view_file(int, int *, DIRTAB[], int, GMODE[]);
int  grav_view_single_file(char *, int, GMODE[]);
int  grav_view_on_screen(char *, GBM *, GBMRGB[], int, GMODE[], int);
int  grav_get_bitmap_width(GBM *);
int  grav_display(char *, int, int, int, int, int);
int  grav_set_palette(GBM *, GBMRGB[]);
int  grav_set_blank_palette(GBM *);
int  grav_verify_mode(int, GMODE[], GBM *);
int  grav_new_mode(int, int *, GMODE[]);
int  grav_get_screen_width(int, GMODE[], GBM *);
int  grav_get_scroll_amount(int *, int *, int *, int *, int, GMODE[], GBM *);
